%
%  version FDM 2D
%

global data

tol = 1e-10;

Nx = data.Nx;
Ny = data.Ny;
nodes_bou = data.nodes_bou;
normal_node = data.normal_node;
[numnp,ndm] = size(xnod);

if data.pre_step
    fdm_assemble_pre;
else
    Indx = data.Indx;
    Indx_inv = data.Indx_inv;
    dx_fwd = data.dx_fwd;
    dy_fwd = data.dy_fwd;
    dx_bkw = data.dx_bkw;
    dy_bkw = data.dy_bkw;
    dx_half = data.dx_half;
    dy_half = data.dy_half;
    dx_cen = data.dx_cen;
    dy_cen = data.dy_cen;
    rowg = data.rowg;
    colg = data.colg;
    coeg = data.coeg;
end

R = zeros(numnp,1);
for jndx=2:Ny-1,

    indx = (2:Nx-1);
    % computation of divided differences to approximate the spatial
    % derivatives

    dfdx_fwd(Indx(indx,jndx),1) = (state(Indx(indx+1,jndx),1)-state(Indx(indx,jndx),1));
    dfdx_bkw(Indx(indx,jndx),1) = (state(Indx(indx,jndx),1)-state(Indx(indx-1,jndx),1));
    dfdx_cen(Indx(indx,jndx),1) = (state(Indx(indx+1,jndx),1)-state(Indx(indx-1,jndx),1));

    dfdy_fwd(Indx(indx,jndx),1) = (state(Indx(indx,jndx+1),1)-state(Indx(indx,jndx),1));
    dfdy_bkw(Indx(indx,jndx),1) = (state(Indx(indx,jndx),1)-state(Indx(indx,jndx-1),1));
    dfdy_cen(Indx(indx,jndx),1) = (state(Indx(indx,jndx+1),1)-state(Indx(indx,jndx-1),1));

    % computation of second derivatives in both directions
    d2fdx2(Indx(indx,jndx),1) = (dfdx_fwd(Indx(indx,jndx),1)./dx_fwd(Indx(indx,jndx),1) - ...
        dfdx_bkw(Indx(indx,jndx),1)./dx_bkw(Indx(indx,jndx),1)) ./ dx_half(Indx(indx,jndx),1);

    d2fdy2(Indx(indx,jndx),1) = (dfdy_fwd(Indx(indx,jndx),1)./dy_fwd(Indx(indx,jndx),1) - ...
        dfdy_bkw(Indx(indx,jndx),1)./dy_bkw(Indx(indx,jndx),1)) ./ dy_half(Indx(indx,jndx),1);

end

% computation of residue and matrix
%rowg = []; colg = []; coeg=[];

for jndx=2:Ny-1,
    R(Indx(indx,jndx),1) = (d2fdx2(Indx(indx,jndx),1) + d2fdy2(Indx(indx,jndx),1)) + ... 
        source(Indx(indx,jndx),1) - rho*Cp/Dt * (state(Indx(indx,jndx),1)-state_old(Indx(indx,jndx),1));
end

%keyboard;pause
% fixations
% fixed or Dirichlet
nodes_fixed = find(fixa(:,1)==1);
R(nodes_fixed,1) = 0;

% computation of some derivatives for boundary nodes, using ghost nodes
% (ficticious)
indx = 1; jndx = (1:Ny);
dx_fwd(Indx(indx,jndx),1) = (xnod(Indx(indx+2,jndx),1)-xnod(Indx(indx+1,jndx),1));
dfdx_fwd(Indx(indx,jndx),1) = (state(Indx(indx+2,jndx),1)-state(Indx(indx,jndx),1));
dfdx(Indx(indx,jndx),1) = dfdx_fwd(Indx(indx,jndx),1)./(2*dx_fwd(Indx(indx,jndx),1));

indx = Nx; jndx = (1:Ny);
dx_bkw(Indx(indx,jndx),1) = (xnod(Indx(indx-1,jndx),1)-xnod(Indx(indx-2,jndx),1));
dfdx_bkw(Indx(indx,jndx),1) = (state(Indx(indx,jndx),1)-state(Indx(indx-2,jndx),1));
dfdx(Indx(indx,jndx),1) = dfdx_bkw(Indx(indx,jndx),1)./(2*dx_bkw(Indx(indx,jndx),1));

indx = (1:Nx); jndx = 1;
dy_fwd(Indx(indx,jndx),1) = (xnod(Indx(indx,jndx+1),2)-xnod(Indx(indx,jndx),2));
dfdy_fwd(Indx(indx,jndx),1) = (state(Indx(indx,jndx+2),1)-state(Indx(indx,jndx),1));
dfdy(Indx(indx,jndx),1) = dfdy_fwd(Indx(indx,jndx),1)./(2*dy_fwd(Indx(indx,jndx),1));

indx = (1:Nx); jndx = Ny;
dy_bkw(Indx(indx,jndx),1) = (xnod(Indx(indx,jndx),2)-xnod(Indx(indx,jndx-1),2));
dfdy_bkw(Indx(indx,jndx),1) = (state(Indx(indx,jndx),1)-state(Indx(indx,jndx-2),1));
dfdy(Indx(indx,jndx),1) = dfdy_bkw(Indx(indx,jndx),1)./(2*dy_bkw(Indx(indx,jndx),1));

% free or Neumann
nodes = m2v(nodes_bou);
nodes_Neuman = find(fixa(nodes,1)==0);
nodes_Neuman = rmdblist(nodes(nodes_Neuman));
ijv = [1,0;0,1];
if isempty(nodes_Neuman)==0
    no = normal_node(nodes_Neuman,:);
    for kn=1:length(nodes_Neuman)
        for kdm=1:ndm
            ii = ijv(kdm,1);jj = ijv(kdm,2);
            if abs(no(kn,kdm))>tol
                nn = kn;
                nodes = nodes_Neuman(kn);
                II = Indx_inv(nodes,1);
                JJ = Indx_inv(nodes,2);
                if II==1
                    R(nodes,1) = R(nodes,1) + no(nn,1).*dfdx(nodes,1) + fixa(nodes,2) ;
                elseif II==Nx
                    R(nodes,1) = R(nodes,1) + no(nn,1).*dfdx(nodes,1) + fixa(nodes,2) ;
                elseif JJ==1
                    R(nodes,1) = R(nodes,1) + no(nn,2).*dfdy(nodes,1) + fixa(nodes,2) ;
                elseif JJ==Ny
                    R(nodes,1) = R(nodes,1) + no(nn,2).*dfdy(nodes,1) + fixa(nodes,2) ;
                end
            end
        end
    end
end

